@@ -17,11 +17,26 @@
 # limitations under the License.
 #
 
+"""
+Upload disk example code.
+
+Requires the qemu-img package for checking file type and virtual size.
+
+Usage:
+
+    upload_disk.py FILE
+"""
+
+from __future__ import print_function
+
+import json
 import logging
 import os
 import ovirtsdk4 as sdk
 import ovirtsdk4.types as types
 import ssl
+import subprocess
+import sys
 import time
 
 from httplib import HTTPSConnection
@@ -32,8 +47,26 @@ except ImportError:
     from urlparse import urlparse
 
 
+# This seems to give the best throughput when uploading from my laptop
+# SSD to a server that drop the data. You may need to tune this on your
+# setup.
+BUF_SIZE = 128 * 1024
+
 logging.basicConfig(level=logging.DEBUG, filename='example.log')
 
+image_path = sys.argv[1]
+image_size = os.path.getsize(image_path)
+
+# Get image info using qemu-img
+
+print("Checking image...")
+
+out = subprocess.check_output(
+    ["qemu-img", "info", "--output", "json", image_path])
+image_info = json.loads(out)
+
+if image_info["format"] not in ("qcow2", "raw"):
+    raise RuntimeError("Unsupported image format %(format)s" % image_info)
 
 # This example will connect to the server and create a new `floating`
 # disk, one that isn't attached to any virtual machine.
@@ -41,6 +74,8 @@ logging.basicConfig(level=logging.DEBUG, filename='example.log')
 # qcow2 disk to the newly created disk in server.
 
 # Create the connection to the server:
+print("Connecting...")
+
 connection = sdk.Connection(
     url='https://engine40.example.com/ovirt-engine/api',
     username='admin@internal',
@@ -67,16 +102,23 @@ system_service = connection.system_service()
 #
 # 3. The disk initial size must be bigger or the same as the size of the data
 #    you will upload.
-initial_size = 1 * 2**30
-provisioned_size = 10 * 2**30
+
+print("Creating disk...")
+
+if image_info["format"] == "qcow2":
+    disk_format = types.DiskFormat.COW
+else:
+    disk_format = types.DiskFormat.RAW
+
 disks_service = connection.system_service().disks_service()
 disk = disks_service.add(
     disk=types.Disk(
-        name='mydisk',
-        description='My disk',
-        format=types.DiskFormat.COW,
-        provisioned_size=provisioned_size,
-        initial_size=initial_size,
+        name=os.path.basename(image_path),
+        description='Uploaded disk',
+        format=disk_format,
+        initial_size=image_size,
+        provisioned_size=image_info["virtual-size"],
+        sparse=disk_format == types.DiskFormat.COW,
         storage_domains=[
             types.StorageDomain(
                 name='data'
@@ -94,6 +136,8 @@ while True:
     if disk.status == types.DiskStatus.OK:
         break
 
+print("Creating transfer session...")
+
 # Get a reference to the service that manages the image
 # transfer that was added in the previous step:
 transfers_service = system_service.image_transfers_service()
@@ -116,10 +160,7 @@ while transfer.phase == types.ImageTransferPhase.INITIALIZING:
     time.sleep(1)
     transfer = transfer_service.get()
 
-# Set needed headers for uploading:
-upload_headers = {
-    'Authorization': transfer.signed_ticket,
-}
+print("Uploading image...")
 
 # At this stage, the SDK granted the permission to start transferring the disk, and the
 # user should choose its preferred tool for doing it - regardless of the SDK.
@@ -137,34 +178,73 @@ proxy_connection = HTTPSConnection(
     context=context,
 )
 
-path = "/path/to/disk.qcow2"
-MiB_per_request = 8
-with open(path, "rb") as disk:
-    size = os.path.getsize(path)
-    chunk_size = 1024 * 1024 * MiB_per_request
+# Send the request head. Note the following:
+#
+# - We must send the Authorzation header with the signed ticket received
+#   from the transfer service.
+#
+# - the server requires Content-Range header even when sending the
+#   entire file.
+#
+# - the server requires also Content-Length.
+#
+
+proxy_connection.putrequest("PUT", proxy_url.path)
+proxy_connection.putheader('Authorization', transfer.signed_ticket)
+proxy_connection.putheader('Content-Range',
+                           "bytes %d-%d/%d" % (0, image_size - 1, image_size))
+proxy_connection.putheader('Content-Length', "%d" % (image_size,))
+proxy_connection.endheaders()
+
+# Send the request body. Note the following:
+#
+# - we must send the number of bytes we promised in the Content-Range
+#   header.
+#
+# - we must extend the session, otherwise it will expire and the upload
+#   will fail.
+
+start = last_progress = last_extend = time.time()
+
+with open(image_path, "rb") as disk:
     pos = 0
-    while pos < size:
-        # Extend the transfer session.
-        transfer_service.extend()
-        # Set the content range, according to the chunk being sent.
-        upload_headers['Content-Range'] = "bytes %d-%d/%d" % (pos, min(pos + chunk_size, size) - 1, size)
-        # Perform the request.
-        proxy_connection.request(
-            'PUT',
-            proxy_url.path,
-            disk.read(chunk_size),
-            headers=upload_headers,
-        )
-        # Print response
-        r = proxy_connection.getresponse()
-        print r.status, r.reason, "Completed", "{:.0%}".format(pos / float(size))
-        # Continue to next chunk.
-        pos += chunk_size
+    while pos < image_size:
+        # Send the next chunk to the proxy.
+        to_read = min(image_size - pos, BUF_SIZE)
+        chunk = disk.read(to_read)
+        if not chunk:
+            transfer_service.pause()
+            raise RuntimeError("Unexpected end of file at pos=%d" % pos)
 
+        proxy_connection.send(chunk)
+        pos += len(chunk)
+        now = time.time()
 
-print "Completed", "{:.0%}".format(pos / float(size))
-# Finalize the session.
+        # Report progress every 10 seconds.
+        if now - last_progress > 10:
+            print("Uploaded %.2f%%" % (float(pos) / image_size * 100))
+            last_progress = now
+
+        # Extend the transfer session once per minute.
+        if now - last_extend > 60:
+            transfer_service.extend()
+            last_extend = now
+
+# Get the response
+response = proxy_connection.getresponse()
+if response.status != 200:
+    transfer_service.pause()
+    print("Upload failed: %s %s" % (response.status, response.reason))
+    sys.exit(1)
+
+elapsed = time.time() - start
+
+print("Uploaded %.2fg in %.2f seconds (%.2fm/s)" % (
+      image_size / float(1024**3), elapsed, image_size / 1024**2 / elapsed))
+
+print("Finalizing transfer session...")
+# Successful cleanup
 transfer_service.finalize()
-
-# Close the connection to the server:
 connection.close()
+
+print("Upload completed successfully")
