@@ -60,6 +60,46 @@ class Error(Exception):
         self.fault = fault
 
 
+class AuthError(Error):
+    """
+    This class of error indicates that an authentiation or authorization
+    problem happenend, like incorrect user name, incorrect password, or
+    missing permissions.
+    """
+    pass
+
+
+class ConnectionError(Error):
+    """
+    This class of error indicates that the name of the server or the name of
+    the proxy can't be resolved to an IP address, or that the connection
+    can't be stablished because the server is down or unreachable.
+
+    Note that for this class of error the `code` and `fault` attributes will
+    always be empty, as no response from the server will be available to
+    populate them.
+    """
+    pass
+
+
+class NotFoundError(Error):
+    """
+    This class of error indicates that an object can't be found.
+    """
+    pass
+
+
+class TimeoutError(Error):
+    """
+    This class of error indicates that an operation timed out.
+
+    Note that for this class of error the `code` and `fault` attributes will
+    always be empty, as no response from the server will be available to
+    populate them.
+    """
+    pass
+
+
 class List(list):
     """
     This is the base class for all the list types of the SDK. It contains the
@@ -327,14 +367,7 @@ class Connection(object):
         """
 
         with self._curl_lock:
-            try:
-                return self.__send(request)
-            except pycurl.error as e:
-                six.reraise(
-                    Error,
-                    Error("Error while sending HTTP request", e),
-                    sys.exc_info()[2]
-                )
+            return self.__send(request)
 
     def authenticate(self):
         """
@@ -343,14 +376,16 @@ class Connection(object):
         revoked when the connection is closed, unless the `logout` parameter of
         the `close` method is `False`.
         """
-        if self._sso_token is None:
-            self._sso_token = self._get_access_token()
-        return self._sso_token
+        try:
+            if self._sso_token is None:
+                self._sso_token = self._get_access_token()
+            return self._sso_token
+        except pycurl.error as e:
+            self.__parse_error(e)
 
     def __send(self, request):
         # Create SSO token if needed:
-        if self._sso_token is None:
-            self._sso_token = self._get_access_token()
+        self.authenticate()
 
         # Init curl easy:
         curl = pycurl.Curl()
@@ -457,7 +492,10 @@ class Connection(object):
 
     def wait(self, context, failed_auth=False):
         with self._curl_lock:
-            return self.__wait(context, failed_auth)
+            try:
+                return self.__wait(context, failed_auth)
+            except pycurl.error as e:
+                self.__parse_error(e)
 
     def __wait(self, context, failed_auth=False):
         while True:
@@ -531,10 +569,11 @@ class Connection(object):
             sso_response = sso_response[0]
 
         if 'error' in sso_response:
-            raise Error(
+            sso_error = self._get_sso_error(sso_response)
+            raise AuthError(
                 'Error during SSO revoke %s : %s' % (
-                    sso_response['error_code'],
-                    sso_response['error']
+                    sso_error[0],
+                    sso_error[1]
                 )
             )
 
@@ -581,15 +620,25 @@ class Connection(object):
             sso_response = sso_response[0]
 
         if 'error' in sso_response:
-            raise Error(
+            sso_error = self._get_sso_error(sso_response)
+            raise AuthError(
                 'Error during SSO authentication %s : %s' % (
-                    sso_response['error_code'],
-                    sso_response['error']
+                    sso_error[0],
+                    sso_error[1]
                 )
             )
 
         return sso_response[self._sso_token_name]
 
+    def _get_sso_error(self, sso_response):
+        # OpenId define `error_description` field, OAuth doesn't:
+        if 'error_description' in sso_response:
+            sso_error = (sso_response.get('error'), sso_response.get('error_description'))
+        else:
+            sso_error = (sso_response.get('error_code'), sso_response.get('error'))
+
+        return sso_error
+
     def _get_sso_response(self, url, params=''):
         """
         Perform SSO request and return response body data.
@@ -691,6 +740,10 @@ class Connection(object):
         try:
             self.system_service().get()
             return True
+        except Error:
+            if raise_exception:
+                six.reraise(*sys.exc_info())
+            return False
         except Exception as exception:
             if raise_exception:
                 raise Error(exception)
@@ -748,7 +801,7 @@ class Connection(object):
         """
 
         # Revoke access token:
-        if logout:
+        if logout and self._sso_token is not None:
             self._revoke_access_token()
 
         # Release resources used by the cURL handle:
@@ -864,6 +917,20 @@ class Connection(object):
         # Return the response:
         return response
 
+    def __parse_error(self, error):
+        e_code = error.args[0]
+        clazz = Error
+        error_msg = "Error while sending HTTP request: {}".format(error)
+
+        if e_code in [
+            pycurl.E_COULDNT_CONNECT, pycurl.E_COULDNT_RESOLVE_HOST
+        ]:
+            clazz = ConnectionError
+        elif e_code == pycurl.E_OPERATION_TIMEDOUT:
+            clazz = TimeoutError
+
+        six.reraise(clazz, clazz(error_msg), sys.exc_info()[2])
+
     def _get_header_value(self, headers, name):
         """
         Return header value by its name.
